@@ -45,6 +45,74 @@
 
 static GList *printers = NULL;
 
+/*******************************************/
+/* CUPS requests                           */
+/* (widely inspired from the libgnomecups) */
+/*******************************************/
+static ipp_t *
+cups_request_new (int operation_id)
+{
+  ipp_t *request;
+  cups_lang_t *language;
+
+  language = cupsLangDefault ();
+  request = ippNew ();
+  request->request.op.operation_id = operation_id;
+  request->request.op.request_id = 1;
+
+  ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_CHARSET, "attributes-charset", NULL, "utf-8");
+
+  ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE, "attributes-natural-language", NULL, language->language);
+  cupsLangFree (language);
+
+  return request;
+}
+
+static ipp_t *
+cups_request_new_for_printer (int operation_id, const gchar * printer)
+{
+  ipp_t *request;
+  gchar *printer_uri;
+
+  g_return_val_if_fail (printer, NULL);
+
+  printer_uri = g_strdup_printf ("ipp://%s/printers/%s", cupsServer (), printer);
+  request = cups_request_new (operation_id);
+
+  ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_URI, "printer-uri", NULL, printer_uri);
+  g_free (printer_uri);
+
+  return request;
+}
+
+static ipp_t *
+cups_request_execute (ipp_t * request, const gchar * path)
+{
+  http_t *HTTP = NULL;
+  ipp_status_t status;
+
+  HTTP = httpConnectEncrypt (cupsServer (), ippPort (), cupsEncryption ());
+  if (!HTTP) {
+    ippDelete (request);
+    g_warning ("Unable to connect CUPS server");
+    return NULL;
+  }
+
+  request = cupsDoRequest (HTTP, request, path);
+  httpClose (HTTP);
+  status = cupsLastError ();
+  if (!request) {
+    g_warning ("CUPS server couldn't execute request");
+    return NULL;
+  }
+
+  if (status > IPP_OK_CONFLICT)
+    g_warning ("failed request with status %d", status);
+
+  return request;
+}
+
+
 /***************/
 /* Main window */
 /***************/
@@ -61,32 +129,20 @@ get_printers_list ()
   if (num_dests > 0) {
     for (i = 0; i < num_dests; i++) {
       Printer *printer;
-      http_t *HTTP;
-      ipp_t *request = ippNew ();
-      cups_lang_t *lang = cupsLangDefault ();
+      ipp_t *request;
       ipp_attribute_t *attr;
-      char buf[1024];
 
       printer = g_new0 (Printer, 1);
-      printer->name = g_strdup (dests[i].name);
+
+      if (dests[i].instance)
+	printer->name = g_strdup_printf ("%s/%s", dests[i].name, dests[i].instance);
+      else
+	printer->name = g_strdup (dests[i].name);
+
       list = g_list_append (list, printer);
 
-      request->request.op.request_id = 1;
-      ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_CHARSET, "attributes-charset", NULL, cupsLangEncoding (lang));
-      ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE, "attributes-natural-language", NULL, lang->language);
-
-      sprintf (buf, "ipp://localhost:631/printers/%s", dests[i].name);
-      ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_URI, "printer-uri", NULL, buf);
-      request->request.op.operation_id = IPP_GET_PRINTER_ATTRIBUTES;
-
-      HTTP = httpConnect ("localhost", 631);
-      if (!HTTP) {
-        ippDelete (request);
-        continue;
-      }
-
-      request = cupsDoRequest (HTTP, request, "/printers/");
-      httpClose (HTTP);
+      request = cups_request_new_for_printer (IPP_GET_PRINTER_ATTRIBUTES, dests[i].name);
+      request = cups_request_execute (request, "/printers/");
 
       if (!request)
         continue;
@@ -115,7 +171,8 @@ get_printers_list ()
 
       ippDelete (request);
     }
-  }
+  } else
+    g_warning ("no printer in the list, may be the CUPS server isn't running or you haven't configured any printer");
 
   cupsFreeDests (num_dests, dests);
 
@@ -210,9 +267,10 @@ G_MODULE_EXPORT GList * printing_system_get_jobs_list_impl (const gchar * printe
       local_time = localtime (&(jobs[i].processing_time));
       strftime (string_processing_time, sizeof (string_processing_time), "%H:%M:%S", local_time);
       job->processing_time = g_strdup (string_processing_time);
-
-      list = g_list_append (list, job);
     }
+    
+    list = g_list_append (list, job);
+      
   }
 
   cupsFreeJobs (num_jobs, jobs);
@@ -249,6 +307,13 @@ printing_system_properties_dialog_run_impl (const gchar * printer)
 /****************/
 /* Print dialog */
 /****************/
+static void
+printer_name_free (gchar *printer_name, gpointer user_data)
+{
+  if (printer_name)
+    g_free (printer_name);
+}
+
 static gboolean
 destroy_print_dialog_cb (GtkWidget * widget, GdkEvent * event, gpointer user_data)
 {
@@ -274,9 +339,12 @@ printing_system_print_dialog_customize_impl (PrintDlg * dialog)
 
   printer_el = g_list_first (printers);
   while (printer_el) {
+    gchar *list_entry;
+
     printer = printer_el->data;
 
-    list = g_list_append (list, printer->alias);
+    list_entry = g_strdup_printf ("%s (%s)", printer->alias, printer->name);
+    list = g_list_append (list, list_entry);
 
     printer_el = g_list_next (printer_el);
   }
@@ -293,8 +361,13 @@ printing_system_print_dialog_customize_impl (PrintDlg * dialog)
 
   /* select default printer */
   printer = printing_system_get_default_printer_impl (printers);
-  if (printer)
-    gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (dialog->customization)->entry), printer->alias);
+  if (printer) {
+    gchar *printer_name;
+
+    printer_name = g_strdup_printf ("%s (%s)", printer->alias, printer->name);
+    gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (dialog->customization)->entry), printer_name);
+    g_free (printer_name);
+  }
 
   if (g_list_length (list) == 0) {
     gtk_widget_set_sensitive (GTK_WIDGET (dialog->customization), FALSE);
@@ -306,6 +379,7 @@ printing_system_print_dialog_customize_impl (PrintDlg * dialog)
 
   gtk_box_reorder_child (GTK_BOX (GTK_DIALOG (dialog->dialog)->vbox), hbox, 0);
 
+  g_list_foreach (list, (GFunc) printer_name_free, NULL);
   g_list_free (list);
 }
 
@@ -315,15 +389,33 @@ printing_system_print_dialog_customize_impl (PrintDlg * dialog)
 G_MODULE_EXPORT gboolean printing_system_print_file_impl (PrintDlg * dialog, const gchar * file)
 {
   gchar *base_name = NULL;
-  Printer *printer;
+  const gchar* printer_selected;
+  gchar *printer_name = NULL;
+  gchar *printer_instance = NULL;
+  int i, j, length;
+  int num_dests;
+  cups_dest_t *dests;
+  cups_dest_t *dest;
   gboolean ret = FALSE;
 
   base_name = g_path_get_basename (dialog->original_name);
 
-  printer =
-    printer_lookup_byalias (printers, gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (dialog->customization)->entry)));
+  /* extract printer name and instance name */
+  printer_selected = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (dialog->customization)->entry));
+  length = strlen (printer_selected);
 
-  if (cupsPrintFile (printer->name, file, base_name, 0, NULL)) {
+  for (i = length; i >= 0 && printer_selected[i] != '('; i--);
+  i++; /* remove last '(' */
+  for (j = i; j < length && printer_selected[j] != '/' && printer_selected[j] != ')'; j++);
+  printer_name = g_strndup (&printer_selected[i], j - i);
+  j++; /* remove the '/' */
+  if (j < length)
+    printer_instance = g_strndup (&printer_selected[j], length - j - 1);
+
+  num_dests = cupsGetDests(&dests);
+  dest = cupsGetDest(printer_name, printer_instance, num_dests, dests);
+
+  if (cupsPrintFile (printer_name, file, base_name, dest->num_options, dest->options)) {
     unlink (file);
     ret = TRUE;
   }
@@ -331,6 +423,9 @@ G_MODULE_EXPORT gboolean printing_system_print_file_impl (PrintDlg * dialog, con
     xfce_err (ippErrorString (cupsLastError ()));
   }
 
+  cupsFreeDests(num_dests, dests);
+  g_free (printer_name);
+  g_free (printer_instance);
   g_free (base_name);
 
   return ret;
