@@ -73,6 +73,8 @@ show_printer_queue (const gchar * printer_name)
   GtkCellRenderer *cell_icon, *cell_name;
   GtkTreeViewColumn *column_name;
 
+  GdkPixbuf *icon = NULL;
+
   /* Create window */
   queuedlg = g_new0 (QueueDlg, 1);
 
@@ -86,6 +88,11 @@ show_printer_queue (const gchar * printer_name)
   title = g_strdup_printf ("%s - %s", _("Queue Manager"), (printer_lookup_byname (printers_list, printer_name))->alias);
   gtk_window_set_title (GTK_WINDOW (queuedlg->window), title);
   g_free (title);
+  icon = xfce_icon_theme_load (queuedlg->icontheme, "xfprint", 48);
+  if (icon) {
+    gtk_window_set_icon (GTK_WINDOW (queuedlg->window), icon);
+    g_object_unref (icon);
+  }
   gtk_window_set_default_size (GTK_WINDOW (queuedlg->window), 450, 150);
   gtk_window_add_accel_group (GTK_WINDOW (queuedlg->window), queuedlg->accel_group);
   gtk_window_set_position (GTK_WINDOW (queuedlg->window), GTK_WIN_POS_NONE);
